@@ -1,7 +1,7 @@
 package Rope;
 
 use 5.006; use strict; use warnings;
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 use Rope::Object;
 my (%META, %PRO);
 our @ISA;
@@ -60,10 +60,10 @@ BEGIN {
 						};
 						$props{properties}{$prop}{$_} = sub {
 							my (@params) = @_;
-							for (my $i = scalar @{$mod} - 1; $i >= 1; $i--) {
+							for (my $i = 0; $i < scalar @{$mod} - 1; $i++) {
 								@params = ($cb->($mod->[$i], @params));
 							}
-							return $cb->($mod->[0], @params);
+							return $cb->($mod->[-1], @params);
 						};
 					}
 				}
@@ -128,6 +128,7 @@ BEGIN {
 		requires => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] && $_[0] eq $caller;
 				my (@requires) = @_;
 				$META{$caller}{requires}{$_}++ for (@requires);
 			};
@@ -135,6 +136,7 @@ BEGIN {
 		function => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my ($prop, @options) = @_;
 				$prop = shift @options if ( @options > 1 );
 				$PRO{set_prop}(
@@ -151,6 +153,7 @@ BEGIN {
 		properties => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my (@properties) = @_;
 				while (@properties) {
 					my ($prop, $options) = (shift @properties, shift @properties);
@@ -178,6 +181,7 @@ BEGIN {
 		property => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my ($prop, @options) = @_;
 				if (scalar @options % 2) {
 					$prop = shift @options;
@@ -192,6 +196,7 @@ BEGIN {
 		prototyped => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my (@proto) = @_;
 				while (@proto) {
 					my ($prop, $value) = (shift @proto, shift @proto);
@@ -210,6 +215,7 @@ BEGIN {
 		with => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my (@withs) = @_;
 				for my $with (@withs) {
 					if (!$META{$with}) {
@@ -251,6 +257,7 @@ BEGIN {
 		extends => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my (@extends) = @_;
 				for my $extend (@extends) {
 					if (!$META{$extend}) {
@@ -294,6 +301,7 @@ BEGIN {
 		before => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my ($prop, $cb) = @_;
 				push @{$META{$caller}{properties}{$prop}{before}}, $cb;
 			};
@@ -301,6 +309,7 @@ BEGIN {
 		around => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my ($prop, $cb) = @_;
 				push @{$META{$caller}{properties}{$prop}{around}}, $cb;
 			};
@@ -308,8 +317,9 @@ BEGIN {
 		after => sub {
 			my ($caller) = shift;
 			return sub {
+				shift @_ if $_[0] eq $caller;
 				my ($prop, $cb) = @_;
-				push @{$META{$caller}{properties}{$prop}{before}}, $cb;
+				push @{$META{$caller}{properties}{$prop}{after}}, $cb;
 			};
 		},
 		locked => sub {
@@ -483,7 +493,7 @@ Rope - Tied objects
 
 =head1 VERSION
 
-Version 0.20
+Version 0.21
 
 =cut
 
